#include "common.h"

double normal(double mu, double sigma);


void CHANNEL::Set_channel(double _distance, bool _Indoor, int _bs_idx, int _ms_idx, LOCATION _bs_location, LOCATION _ms_location)
{
	distance = _distance;
	Indoor = _Indoor;

	self_bs_idx = _bs_idx;
	self_ms_idx = _ms_idx;
	bs_location = _bs_location;
	ms_location = _ms_location;

	//////////////////////////////////////////////
	///////// Large Scale Parameter///////////////
	//////////////////////////////////////////////
	Set_LOS_Prob();
	Set_PATHLOSS();
	Set_SHADOWFADING();
	

	//////////////////////////////////////////////
	/// Get Small Scale parameter/////////////////
	//////////////////////////////////////////////
	/*
	Set_Channel_Parameters();
	Set_DELAY(); 
	Set_POWER();
	Find_Strong2Clusters();

	if (CH_CAL == 0 || CH_CAL == 2)       // For Pre-Calculate , ̸ ϴ ä  LOS NLOS 츦  Ͽ 
	{
		Set_AOAAOD_LOS();
		Set_ZOAZOD_LOS();

		Set_AOAAOD_NLOS();
		Set_ZOAZOD_NLOS();

		Set_AOAAOD();
		Set_ZOAZOD();
	}
	else
	{
		Set_AOAAOD();
		Set_ZOAZOD();
	}

	Set_InitialPhase(); 
	Set_SUBCLUSTER();

	Set_RMS_delay_spread(); // for calibration
	Set_circular_angle_spread();
	*/

}


void CHANNEL::Set_LOS_Prob()
{

	//////////////////////////////
	///
	/// LOS Prob
	///
	//////////////////////////////////

	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 46


	if (TYPE == 11) // InH
	{
		if (distance <= 5)
		{
			LOS_prob = 1;
		}
		else if (distance > 5. && distance < 49.)
		{
			LOS_prob = exp(-(distance - 5.) / 70.8);
		}
		else if (distance >= 49.)
		{
			LOS_prob = exp(-(distance - 49.) / 211.7) * 0.54;
		}

		double P = randnum.u();
		if (P < LOS_prob)
		{
			LOS = true;
		}
		else
		{
			LOS = false;
		}

	}
	else if (TYPE == 12) // Dense_Urban UMa
	{


		double n = 0;
		double N = floor((5. * randnum.u()) + 4.); // 4 ~ 8
		n = floor(((N)* randnum.u()) + 1.); // 1 ~ N

		ms_height_in = 3 * (n - 1) + 1.5;


		if (Indoor == false) // outdoor
		{
			double C = 0;

			if (ms_height_out <= 13)
			{
				C = 0;
			}
			else
			{
				C = pow((ms_height_out - 13.) / 10., 1.5);
			}

			if (distance <= 18)
			{
				LOS_prob = 1;
			}
			else if (distance > 18)
			{
				LOS_prob = ((18. / distance) + exp(-1 * distance / 63.) * (1 - (18. / distance))) * (1 + C * 5/4 * pow(distance/100,3) * exp(-1 * distance / 150.));
			}

			double P = randnum.u();

			if (P < LOS_prob)
			{
				LOS = true;
			}
			else
			{
				LOS = false;
			}
		}
		else if (Indoor == true)  /// Indoor
		{

			double d_in;
			double d_out;

			d_in = 25. * randnum.u();  //// UMa, UMi = 0~25, RMa = 0~10 
			d_out = distance - d_in;

			double C = 0;

			if (ms_height_in <= 13)
			{
				C = 0;
			}
			else
			{
				C = pow((ms_height_in - 13.) / 10., 1.5);
			}

			if (d_out <= 18)
			{
				LOS_prob = 1;
			}
			else if (d_out > 18)
			{
				LOS_prob = ((18. / d_out) + exp(-1 * d_out / 63.) * (1 - (18. / d_out))) * (1 + C * 5 / 4 * pow(d_out / 100, 3) * exp(-1 * d_out / 150.));
			}

			double P = randnum.u();
			if (P < LOS_prob)
			{
				LOS = true;
			}
			else
			{
				LOS = false;
			}

		}
	}
	else if (TYPE == 13) // RMa
	{
		if (Indoor == false) // outdoor
		{

			if (distance <= 10)
			{
				LOS_prob = 1;
			}
			else if (distance > 10.)
			{
				LOS_prob = exp(-1 * (distance - 10.) / 1000.);
			}

			double P = randnum.u();
			if (P < LOS_prob)
			{
				LOS = true;
			}
			else
			{
				LOS = false;
			}

		}
		else if (Indoor == true)  /// Indoor
		{

			double d_in;
			double d_out;

			d_in = MIN(10. * randnum.u(), 10. * randnum.u());  //// UMa, UMi = 0~25, RMa = 0~10 
			d_out = distance - d_in;

			if (distance <= 10)
			{
				LOS_prob = 1;
			}
			else if (distance > 10.)
			{
				LOS_prob = exp(-1*(d_out - 10.) / 1000.);
			}

			double P = randnum.u();
			if (P < LOS_prob)
			{
				LOS = true;
			}
			else
			{
				LOS = false;
			}

		}
	}
	
}

void CHANNEL::Set_PATHLOSS()
{
	////////////////////////////// PathLoss ////////////////////////////////
	//PathLoss
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 39 ~ 


	if (TYPE == 11) // InH
	{
		double distance_3d;
		distance_3d = sqrt(distance * distance + (bs_height - ms_height) * (bs_height - ms_height)); //3D distance

		if (Channel_Model_Type == 0) /// Channel Model A
		{
			if (LOS == 1)  /// LOS
			{
				if (Frequency <= 6 * pow(10,9))  ///// 0.5GHz <= fc <= 6GHz
				{
					sigma_SF = 3.;
					pathloss = 16.9 * log10(distance_3d) + 32.8 + 20 * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
				}
				else  // 6GHz < fc <= 100 GHz
				{
					sigma_SF = 3.;
					pathloss = 32.4 + 17.3 * log10(distance_3d) + 20 * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
				}
			}
			else if (LOS == 0)  // NLOS
			{
				if (Frequency <= 6 * pow(10,9))  ///// 0.5GHz <= fc <= 6GHz
				{
					sigma_SF = 4.;
					pathloss = 43.3 * log10(distance_3d) + 11.5 + 20 * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
				}
				else  // 6GHz < fc <= 100 GHz
				{
					sigma_SF = 8.03;
					pathloss = 43.3 * log10(distance_3d) + 11.5 + 20 * log10(Frequency / 1000000000.) + 4 * randnum.n();
					pathloss_2 = 38.3 * log10(distance_3d) + 17.3 + 24.9 * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
					pathloss = MAX(pathloss, pathloss_2);
				}
			}
		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{
			if (LOS == 1)  /// LOS
			{
				sigma_SF = 3.;
				pathloss = 17.3 * log10(distance_3d) + 32.4 + 20 * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();

			}
			else if (LOS == 0)  // NLOS
			{
				sigma_SF = 8.03;
				pathloss = 17.3 * log10(distance_3d) + 32.4 + 20 * log10(Frequency / 1000000000.) + 3 * randnum.n();
				pathloss_2 = 38.3 * log10(distance_3d) + 17.3 + 24.9 * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
				pathloss = MAX(pathloss, pathloss_2);
			}
		}
	}
	else if (TYPE == 12) // Dense_Urban  ///// 켱 UMa 
	{
		double n = 0;
		double N = floor((5. * randnum.u()) + 4.); // 4 ~ 8
		n = floor(((N)* randnum.u()) + 1.); // 1 ~ N

		ms_height_in = 3 * (n - 1) + 1.5;

		double ms_height_in_channel = 0;

		if (Indoor == 1) /// Indoor
		{
			ms_height_in_channel = ms_height_in;
		}
		else if (Indoor == 0) // Outdoor
		{
			ms_height_in_channel = ms_height_out;
		}


		double g;
		double C;
		double hE;

		if (distance > 18.)
		{
			g = (5/4) * pow((distance / 100) , 3) * exp(-1 * distance / 150);
		}
		else
		{
			g = 0;
		}

		if (ms_height_in_channel < 13)
		{
			C = 0;
		}
		else
		{
			C = pow((ms_height_in_channel - 13.) / 10., 1.5) * g;
		}


		if (randnum.u() < (1/ (1 + C)))
		{
			hE = 1;

		}
		else
		{

			hE = floor((ms_height_in_channel - 1.5 - 9.) / 3. * randnum.u()) * 3. + 12.;
		}

		double d_BP = 4 * (ms_height_in_channel - hE)*(bs_height - hE)*Frequency / light_speed;
		double distance_3d;
		distance_3d = sqrt(distance * distance + (bs_height - ms_height_in_channel) * (bs_height - ms_height_in_channel)); //3D distance

		if (Channel_Model_Type == 0) /// Channel Model A
		{
			if (LOS == 1)
			{
				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
				}
				else if (d_BP < distance)
				{
					sigma_SF = 4.;
					pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2)) + sigma_SF * randnum.n();
				}
			}
			else if (LOS == 0)
			{
				if (Frequency <= 6 * pow(10,9))  ///// 0.5GHz <= fc <= 6GHz
				{
					if (distance <= d_BP)
					{
						sigma_SF = 4.;
						pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
					}
					else if (d_BP < distance)
					{
						sigma_SF = 4.;
						pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2)) + sigma_SF * randnum.n();
					}

					pathloss_2 = 161.04 - 7.1 * log10(20) + 7.5*log10(20) - (24.37 - 3.7*(20 / bs_height)*(20 / bs_height))*log10(bs_height) + (43.42 - 3.1*log10(bs_height))*(log10(distance_3d) - 3) + 20 * log10(Frequency / 1000000000.) - (3.2*(pow(log10(17.625), 2)) - 4.97) - 0.6*(ms_height_in_channel - 1.5) + 6 * randnum.n();

					pathloss = MAX(pathloss, pathloss_2);


				}
				else  // 6GHz < fc <= 100 GHz
				{
					if (distance <= d_BP)
					{
						sigma_SF = 4;
						pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
					}
					else if (d_BP < distance)
					{
						sigma_SF = 4;
						pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2)) + sigma_SF * randnum.n();
					}

					pathloss_2 = 13.54 + 39.08*log10(distance_3d) + 20 * log10(Frequency / 1000000000.) - 0.6*(ms_height_in_channel - 1.5) + 6 * randnum.n();

					pathloss = MAX(pathloss, pathloss_2);
				}
			}

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

			if (LOS == 1)
			{
				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
				}
				else if (d_BP < distance)
				{
					sigma_SF = 4.;
					pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2)) + sigma_SF * randnum.n();
				}
			}
			else if (LOS == 0)
			{
				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss = 28. + 22. * log10(distance_3d) + 20. * log10(Frequency / 1000000000.) + sigma_SF * randnum.n();
				}
				else if (d_BP < distance)
				{
					sigma_SF = 4.;
					pathloss = 40. * log10(distance_3d) + 28 + 20. * log10(Frequency / 1000000000.) - 9 * log10(pow(d_BP, 2) + pow((bs_height - ms_height_in_channel), 2)) + sigma_SF * randnum.n();
				}

				pathloss_2 = 13.54 + 39.08*log10(distance_3d) + 20 * log10(Frequency / 1000000000.) - 0.6*(ms_height_in_channel - 1.5) + 6 * randnum.n();

				pathloss = MAX(pathloss, pathloss_2);
			}
		}

	}
	else if (TYPE == 13) // RMa
	{

		double ms_height_in_channel = 0;

		if (Indoor == 1) /// Indoor
		{
			ms_height_in_channel = ms_height_in;
		}
		else if (Indoor == 0) // Outdoor
		{
			ms_height_in_channel = ms_height_out;
		}

		double d_BP = 2 * pi * (ms_height_in_channel)*(bs_height)*Frequency / light_speed;
		double distance_3d;
		distance_3d = sqrt(distance * distance + (bs_height - ms_height_in_channel) * (bs_height - ms_height_in_channel)); //3D distance

		if (Channel_Model_Type == 0) /// Channel Model A
		{
			double hh = 5.;
			double WW = 20.;

			if (LOS == 1)
			{
				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss = 20 * log10(40 * pi*distance_3d*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(distance_3d) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*distance_3d + sigma_SF * randnum.n();
				}
				else if (d_BP < distance && distance < 21000.)
				{
					sigma_SF = 6.;
					pathloss_2 = 20 * log10(40 * pi*d_BP*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(d_BP) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*d_BP + sigma_SF * randnum.n();

					pathloss = pathloss_2 + 40 * log10(distance_3d / d_BP);
				}
			}
			else if (LOS == 0)
			{
				sigma_SF = 8.;
				pathloss = 161.04 - 7.1*log10(WW) + 7.5*log10(hh) - (24.37 - 3.7*(pow((hh / bs_height), 2)))*log10(bs_height) + (43.42 - 3.1*log10(bs_height))*(log10(distance_3d) - 3) + 20 * log10(Frequency / 1000000000.) - (3.2*(log10(11.75*ms_height_in_channel))*(log10(11.75*ms_height_in_channel)) - 4.97) + sigma_SF * randnum.n();
			}
		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{
			double hh = 5.;
			double WW = 20.;

			if (LOS == 1)
			{
				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss = 20 * log10(40 * pi*distance_3d*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(distance_3d) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*distance_3d + sigma_SF * randnum.n();
				}
				else if (d_BP < distance)
				{
					sigma_SF = 6.;
					pathloss_2 = 20 * log10(40 * pi*d_BP*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(d_BP) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*d_BP + sigma_SF * randnum.n();

					pathloss = pathloss_2 + 40 * log10(distance_3d / d_BP);
				}
			}
			else if (LOS == 0)
			{

				sigma_SF = 8.;
				pathloss = 161.04 - 7.1*log10(WW) + 7.5*log10(hh) - (24.37 - 3.7*(pow((hh / bs_height), 2)))*log10(bs_height) + (43.42 - 3.1*log10(bs_height))*(log10(distance_3d) - 3) + 20 * log10(Frequency / 1000000000.) - (3.2*(log10(11.75*ms_height_in_channel))*(log10(11.75*ms_height_in_channel)) - 4.97) + sigma_SF * randnum.n();

				if (distance <= d_BP)
				{
					sigma_SF = 4.;
					pathloss_2 = 20 * log10(40 * pi*distance_3d*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(distance_3d) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*distance_3d + sigma_SF * randnum.n();
				}
				else if (d_BP < distance)
				{
					sigma_SF = 6.;
					pathloss_2 = 20 * log10(40 * pi*d_BP*(Frequency / 1000000000.) / 3) + MIN(0.03*pow(hh, 1.72), 10)*log10(d_BP) - MIN(0.044*pow(hh, 1.72), 14.77) + 0.002*log10(hh)*d_BP + sigma_SF * randnum.n();

					pathloss_2 = pathloss_2 + 40 * log10(distance_3d / d_BP);
				}
				pathloss = MAX(pathloss, pathloss_2);
			}
		}

	}
}

void CHANNEL::Set_SHADOWFADING()
{

	////// IMT 2020 EVAL Page 39 ~ 

	if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}




}

double normal(double mu, double sigma)
{
	double U1, U2, W, mult;
	static double X1, X2;
	static int call = 0;

	if (call == 1)
	{
		call = !call;
		return (mu + sigma * (double)X2);
	}

	do
	{
		U1 = -1 + randnum.u() * 2;
		U2 = -1 + randnum.u() * 2;
		W = pow(U1, 2) + pow(U2, 2);
	} while (W >= 1 || W == 0);

	mult = sqrt((-2 * log(W)) / W);
	X1 = U1 * mult;
	X2 = U2 * mult;

	call = !call;

	return (mu + sigma * (double)X1);
}

void CHANNEL::Set_Channel_Parameters()
{

	////////////////////////// Parameter setting /////////////////////////////////////////////////////

	if (TYPE == 1)//InH
	{
		if (LOS == 1) ///// LOS
		{
			num_path = 15;
			//num_ray = 20;
			r_tau = 3.6;
			K_factor = normal(7., 4.);
			sigma_tau = pow(10, normal(-7.7, 0.18));

			mu_K_factor = 7.;
			sigma_K_factor = 4.;
			mu_DS = -7.7;
			sigma_DS = 0.18;

			mu_ASD = 1.6;
			sigma_ASD = 0.18;
			mu_ASA = 1.62;
			sigma_ASA = 0.22;
			cluster_ASD = 5;
			cluster_ASA = 8;
			XPR = 11;
			cluster_shadowing = 6;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}


		}
		else if (LOS == 0)   /////////// NLOS
		{

			num_path = 19;
			//num_ray = 20;
			r_tau = 3;
			K_factor = -100;
			sigma_tau = pow(10, normal(-7.41, 0.14));

			mu_K_factor = 0;
			sigma_K_factor = 0;
			mu_DS = -7.41;
			sigma_DS = 0.14;

			mu_ASD = 1.62;
			sigma_ASD = 0.25;
			mu_ASA = 1.77;
			sigma_ASA = 0.16;
			cluster_ASD = 5;
			cluster_ASA = 11;
			XPR = 10;
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}

	}
	else if (TYPE == 2)//UMi
	{

		if (LOS == 1)
		{
			num_path = 12;
			//num_ray = 20;
			r_tau = 3.2;
			K_factor = normal(9, 5);
			sigma_tau = pow(10, normal(-7.19, 0.4));

			mu_K_factor = 9.;
			sigma_K_factor = 5.;
			mu_DS = -7.19;
			sigma_DS = 0.4;

			mu_ASD = 1.2;
			sigma_ASD = 0.43;
			mu_ASA = 1.75;
			sigma_ASA = 0.19;
			cluster_ASD = 3;
			cluster_ASA = 17;
			XPR = 9;
			cluster_shadowing = 6;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
		else if (LOS == 0)
		{

			num_path = 19;
			//num_ray = 20;
			r_tau = 3;
			K_factor = -100;
			sigma_tau = pow(10, normal(-6.89, 0.54));

			mu_K_factor = 0;
			sigma_K_factor = 0;
			mu_DS = -6.89;
			sigma_DS = 0.54;

			mu_ASD = 1.41;
			sigma_ASD = 0.17;
			mu_ASA = 1.84;
			sigma_ASA = 0.15;
			cluster_ASD = 10;
			cluster_ASA = 22;
			XPR = 8;
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
	}
	else if (TYPE == 3)//UMa
	{
		if (LOS == 1)
		{
			num_path = 12;
			//num_ray = 20;
			r_tau = 2.5;
			K_factor = normal(9, 3.5);
			sigma_tau = pow(10, normal(-7.03, 0.66));

			mu_K_factor = 9.;
			sigma_K_factor = 3.5;
			mu_DS = -7.03;
			sigma_DS = 0.66;

			mu_ASD = 1.15;
			sigma_ASD = 0.28;
			mu_ASA = 1.81;
			sigma_ASA = 0.2;
			cluster_ASD = 5;
			cluster_ASA = 11;
			XPR = 8;
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
		else if (LOS == 0)
		{

			num_path = 20;
			//num_ray = 20;
			r_tau = 2.3;
			K_factor = -100;
			sigma_tau = pow(10, normal(-6.44, 0.39));

			mu_K_factor = 0;
			sigma_K_factor = 0;
			mu_DS = -6.44;
			sigma_DS = 0.39;

			mu_ASD = 1.41;
			sigma_ASD = 0.28;
			mu_ASA = 1.87;
			sigma_ASA = 0.11;
			cluster_ASD = 2;
			cluster_ASA = 15;
			XPR = 7;
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
	}
	else if (TYPE == 4) //RMa
	{
		if (LOS == 1)
		{
			num_path = 11;
			//num_ray = 20;
			r_tau = 3.8;
			K_factor = normal(7, 4);
			sigma_tau = pow(10, normal(-7.49, 0.55));

			mu_K_factor = 7.;
			sigma_K_factor = 4.;
			mu_DS = -7.49;
			sigma_DS = 0.55;

			mu_ASD = 0.9;
			sigma_ASD = 0.38;
			mu_ASA = 1.52;
			sigma_ASA = 0.24;
			cluster_ASD = 2;
			cluster_ASA = 3;
			XPR = 12;
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
		else if (LOS == 0)
		{

			num_path = 10;
			//num_ray = 20;
			r_tau = 1.7;
			K_factor = -100;
			sigma_tau = pow(10, normal(-7.43, 0.48));

			mu_K_factor = 0;
			sigma_K_factor = 0;
			mu_DS = -7.43;
			sigma_DS = 0.48;

			mu_ASD = 0.95;
			sigma_ASD = 0.45;
			mu_ASA = 1.52;
			sigma_ASA = 0.13;
			cluster_ASD = 2;
			cluster_ASA = 3;
			XPR = 7;
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}

	}
	else if (TYPE == 5) //SMa
	{
		if (LOS == 1)
		{
			num_path = 15;
			//num_ray = 20;
			r_tau = 2.4;
			K_factor = normal(9, 7);
			sigma_tau = pow(10, normal(-7.23, 0.38));

			mu_K_factor = 9.;
			sigma_K_factor = 7.;
			mu_DS = -7.23;
			sigma_DS = 0.38;

			mu_ASD = 0.78;
			sigma_ASD = 0.12;
			mu_ASA = 1.48;
			sigma_ASA = 0.2;
			cluster_ASD = 5;
			cluster_ASA = 5;
			XPR = 8;
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
		else if (LOS == 0)
		{

			num_path = 14;
			//num_ray = 20;
			r_tau = 1.5;
			K_factor = -100;
			sigma_tau = pow(10, normal(-7.12, 0.33));

			mu_K_factor = 0;
			sigma_K_factor = 0;
			mu_DS = -7.12;
			sigma_DS = 0.33;

			mu_ASD = 0.9;
			sigma_ASD = 0.36;
			mu_ASA = 1.65;
			sigma_ASA = 0.25;
			cluster_ASD = 2;
			cluster_ASA = 10;
			XPR = 4;
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}

	}

	else if (TYPE == 6)//3D UMi
	{

		if (LOS == 1)
		{
			num_path = 12;
			//num_ray = 20;
			r_tau = 3.2;
			K_factor = normal(9, 5);
			sigma_tau = pow(10, normal(-7.19, 0.4));

			mu_K_factor = 9.;
			sigma_K_factor = 5.;
			mu_DS = -7.19;
			sigma_DS = 0.4;

			mu_ASD = 1.2;
			sigma_ASD = 0.43;
			mu_ASA = 1.75;
			sigma_ASA = 0.19;

			mu_ZSA = 0.6;
			sigma_ZSA = 0.16;
			mu_ZSD = MAX(-0.5, (-2.1) * (distance / 1000) + 0.01 * abs(ms_height - bs_height) + 0.75);
			sigma_ZSD = 0.4;

			cluster_ZSA = 7;
			cluster_ASD = 3;
			cluster_ASA = 17;
			
			mu_offset_ZOD = 0;

			XPR = normal(9,3*3);
			cluster_shadowing = 6;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
		else if (LOS == 0)
		{

			num_path = 19;
			//num_ray = 20;
			r_tau = 3;
			K_factor = -100;
			sigma_tau = pow(10, normal(-6.89, 0.54));

			mu_K_factor = 0;
			sigma_K_factor = 0;
			mu_DS = -6.89;
			sigma_DS = 0.54;

			mu_ASD = 1.41;
			sigma_ASD = 0.17;
			mu_ASA = 1.84;
			sigma_ASA = 0.15;
			cluster_ASD = 10;
			cluster_ASA = 22;

			mu_ZSA = 0.88;
			sigma_ZSA = 0.16;
			mu_ZSD = MAX(-0.5, (-2.1) * (distance / 1000) + 0.01 * MAX(ms_height - bs_height , 0) + 0.9);
			sigma_ZSD = 0.6;
			cluster_ZSA = 7;

			mu_offset_ZOD = pow(-10, 0.55 * log10(MAX(10,distance)) + 1.6);

			XPR = normal(8,3*3);
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
	}
	else if (TYPE == 7)//3D UMa
	{
		if (LOS == 1)
		{
			num_path = 12;
			//num_ray = 20;
			r_tau = 2.5;
			K_factor = normal(9, 3.5);
			sigma_tau = pow(10, normal(-7.03, 0.66));

			mu_K_factor = 9.;
			sigma_K_factor = 3.5;
			mu_DS = -7.03;
			sigma_DS = 0.66;

			mu_ASD = 1.15;
			sigma_ASD = 0.28;
			mu_ASA = 1.81;
			sigma_ASA = 0.2;
			cluster_ASD = 5;
			cluster_ASA = 11;

			mu_ZSA = 0.95;
			sigma_ZSA = 0.16;
			mu_ZSD = MAX(-0.5, (-2.1) * (distance/1000) - 0.01 * (ms_height - 1.5) + 0.75);
			sigma_ZSD = 0.4;

			cluster_ZSA = 7;

			mu_offset_ZOD = 0;

			XPR = normal(8, 4*4);
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
		else if (LOS == 0)
		{

			num_path = 20;
			//num_ray = 20;
			r_tau = 2.3;
			K_factor = -100;
			sigma_tau = pow(10, normal(-6.44, 0.39));

			mu_K_factor = 0;
			sigma_K_factor = 0;
			mu_DS = -6.44;
			sigma_DS = 0.39;

			mu_ASD = 1.41;
			sigma_ASD = 0.28;
			mu_ASA = 1.87;
			sigma_ASA = 0.11;
			cluster_ASD = 2;
			cluster_ASA = 15;

			mu_ZSA = 1.26;
			sigma_ZSA = 0.16;
			mu_ZSD = MAX(-0.5, (-2.1) * (distance / 1000) - 0.01 * (ms_height - 1.5) + 0.9);
			sigma_ZSD = 0.49;

			cluster_ZSA = 7;

			mu_offset_ZOD = pow(-10, (-0.62 * log10(MAX(10, distance) + 1.93 - 0.07 * (ms_height - 1.5))));

			XPR = normal(7, 3*3);
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
	}

	else if (TYPE == 8)//3D UMa H
	{
		if (LOS == 1)
		{
			num_path = 12;
			//num_ray = 20;
			r_tau = 2.5;
			K_factor = normal(9, 3.5);
			sigma_tau = pow(10, normal(-7.03, 0.66));

			mu_K_factor = 9.;
			sigma_K_factor = 3.5;
			mu_DS = -7.03;
			sigma_DS = 0.66;

			mu_ASD = 1.15;
			sigma_ASD = 0.28;
			mu_ASA = 1.81;
			sigma_ASA = 0.2;
			cluster_ASD = 5;
			cluster_ASA = 11;

			mu_ZSA = 0.95;
			sigma_ZSA = 0.16;
			mu_ZSD = MAX(-0.5, (-2.1) * (distance / 1000) - 0.01 * (ms_height - 1.5) + 0.75);
			sigma_ZSD = 0.4;
			cluster_ZSA = 7;

			mu_offset_ZOD = 0;

			XPR = normal(8, 4*4);
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
		else if (LOS == 0)
		{

			num_path = 20;
			//num_ray = 20;
			r_tau = 2.3;
			K_factor = -100;
			sigma_tau = pow(10, normal(-6.44, 0.39));

			mu_K_factor = 0;
			sigma_K_factor = 0;
			mu_DS = -6.44;
			sigma_DS = 0.39;

			mu_ASD = 1.41;
			sigma_ASD = 0.28;
			mu_ASA = 1.87;
			sigma_ASA = 0.11;
			cluster_ASD = 2;
			cluster_ASA = 15;

			mu_ZSA = 1.26;
			sigma_ZSA = 0.16;
			mu_ZSD = MAX(-0.5, (-2.1) * (distance / 1000) - 0.01 * (ms_height - 1.5) + 0.9);
			sigma_ZSD = 0.49;
			cluster_ZSA = 7;

			mu_offset_ZOD = pow(-10, (-0.62 * log10(MAX(10, distance) + 1.93 - 0.07 * (ms_height - 1.5))));

			XPR = normal(7, 3*3);
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
	}


	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 58 ~ 

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{
			num_path = 12;
			//num_ray = 20;
			r_tau = 2.5;
			K_factor = normal(9, 3.5);
			sigma_tau = pow(10, normal(-7.03, 0.66));

			mu_K_factor = 9.;
			sigma_K_factor = 3.5;
			mu_DS = -7.03;
			sigma_DS = 0.66;

			mu_ASD = 1.15;
			sigma_ASD = 0.28;
			mu_ASA = 1.81;
			sigma_ASA = 0.2;
			cluster_ASD = 5;
			cluster_ASA = 11;

			mu_ZSA = 0.95;
			sigma_ZSA = 0.16;
			mu_ZSD = MAX(-0.5, (-2.1) * (distance / 1000) - 0.01 * (ms_height - 1.5) + 0.75);
			sigma_ZSD = 0.4;

			cluster_ZSA = 7;

			mu_offset_ZOD = 0;

			XPR = normal(8, 4 * 4);
			cluster_shadowing = 3;

			if (NUM_TX == 1)   ///// 1x2 calibration case
			{
				d_rx = 0.5*Wavelength;
				d_tx = 10 * Wavelength;

			}
			else  //// 4x2 mu mimo case ( eNB Antenna configuration(A) = 4 * wavelength, Antenna configuration(C) = 0.5 * wavelength), rx 0.5 
			{
				d_rx = 0.5*Wavelength;
				d_tx = 0.5 * Wavelength;

			}

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}



}

void CHANNEL::Set_DELAY()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- delay
	///
	/////////////////////////////////////////////////

	D = 0.7705 - 0.0433 * K_factor + 0.0002*K_factor*K_factor + 0.000017*K_factor*K_factor*K_factor;

	delay = new double[MAX_NUM_CLUSTERS];
	delay_LOS = new double[MAX_NUM_CLUSTERS];

	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		delay[i] = 0.;
		delay_LOS[i] = 0.;

	} /// ̸ ϴ Hmκп MAX_NUM_CLUSTER numpath  ʾƼ   // idx num_path Ŭ  delay Ⱚ  ̷  0 ʱȭϵߴ

	for (int i = 0; i < num_path; i++)
	{
		delay[i] = -r_tau * sigma_tau * log(randnum.u());
		
	}

	for (int i = 0; i < num_path; i++)
	{
		double temp;
		if (i == (num_path - 1))
		{
			break;
		}

		for (int j = i + 1; j < num_path; j++)
		{
			temp = delay[i];

			if (delay[j]<temp)
			{
				temp = delay[j];
				delay[j] = delay[i];
				delay[i] = temp;
			}
		}
	} // sort delay,  -> ū

	double temp_delay = delay[0]; //minimum

	for (int i = 0; i < num_path; i++)
	{

		delay[i] = delay[i] - temp_delay;

	}

	//if (LOS == 1)
	 //{
		
		// LOS CASE -> scaled delay for delay spread, not to be used in cluster power generation (M.2135-1 Small scale generation Step 5)
		for (int i = 0; i < num_path; i++)
		{

			delay_LOS[i] = delay[i] / D;
			delay[i] = delay[i];

		}


	//}
	//else
	//{
		//for (int i = 0; i < num_path; i++)
		//{

		//	delay[i] = delay[i];

		//}
	//}


}

void CHANNEL::Set_POWER()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- power
	///
	/////////////////////////////////////////////////

	power = new double[MAX_NUM_CLUSTERS];
	power_LOS = new double[MAX_NUM_CLUSTERS];
	power_NLOS = new double[MAX_NUM_CLUSTERS];   /// LOS, NLOS  Pre-calc ؼ


	double sum_power = 0.;
	double sum_power_LOS = 0.;
	double sum_power_NLOS = 0.;

	double k_R = pow(10, K_factor / 10);

	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		power[i] = 0.;
		power_LOS[i] = 0.;
		power_NLOS[i] = 0.;

	} //// ʱȭ


	for (int i = 0; i < num_path; i++)
	{
		
		power[i] = exp(-delay[i] * (r_tau - 1.) / (r_tau*sigma_tau)) * pow(10., -1 * normal(0, cluster_shadowing) / 10.);
		
		power_LOS[i] = power[i];    // delay LOS cluster power generation
		power_NLOS[i] = power[i];

		sum_power += power[i];

		sum_power_LOS += power_LOS[i];
		sum_power_NLOS += power_NLOS[i];


	}

	if (LOS == 1)
	{
		for (int i = 0; i < num_path; i++)
		{
			power[i] = power[i] / (sum_power*(k_R + 1));
		}
		power[0] = power[0] + k_R / (k_R + 1);
	}
	else if (LOS == 0)
	{
		for (int i = 0; i < num_path; i++)
		{
			power[i] = power[i] / sum_power;
		}
	}




	///// ================================================================== ////
	////  Pre Calc ؼ LOS, NLOS                                ////
	////  ================================================================== ////
	for (int i = 0; i < num_path; i++)
	{
		power_LOS[i] = power_LOS[i] / (sum_power_LOS * (k_R + 1));
	}
	power_LOS[0] = power_LOS[0] + k_R / (k_R + 1);

	for (int i = 0; i < num_path; i++)
	{
		power_NLOS[i] = power_NLOS[i] / sum_power_NLOS;
	}
	////  ================================================================== ////


}

void CHANNEL::Find_Strong2Clusters()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter --  find the 2 strongest clusters
	///
	/////////////////////////////////////////////////

	strongest_power = power[0];
	strongest_power2 = power[1];
	strongest_power_idx = 0;
	strongest_power_idx2 = 1;

	if (power[0] < power[1])
	{
		strongest_power = power[1];
		strongest_power2 = power[0];
		strongest_power_idx = 1;
		strongest_power_idx2 = 0;
	}

	for (int i = 2; i < num_path; i++)
	{
		if (power[i] > strongest_power2)
		{
			if (power[i] > strongest_power)
			{
				strongest_power2 = strongest_power;
				strongest_power_idx2 = strongest_power_idx;
				strongest_power = power[i];
				strongest_power_idx = i;
			}
			else
			{
				strongest_power2 = power[i];
				strongest_power_idx2 = i;
			}
		}
	}
	
}

void CHANNEL::Set_AOAAOD()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- AOA, AOD
	///
	/////////////////////////////////////////////////

	/*
	double sigma_AOD = MIN(pow(10, normal(mu_ASD, sigma_ASD)), 104);
	double sigma_AOA = MIN(pow(10, normal(mu_ASA, sigma_ASA)), 104); ///߸Ȱ , ڵ ãƺ
	*/
	/*
	double sigma_AOD = pow(10, mu_ASD + sigma_ASD * randnum.n());
	double sigma_AOA = pow(10, mu_ASA + sigma_ASA * randnum.n()); /// randnum.n() ´ 𸣰	
	*/

	double sigma_AOD = pow(10, normal(mu_ASD, sigma_ASD));
	double sigma_AOA = pow(10, normal(mu_ASA, sigma_ASA)); 


	/*
	if (TYPE == 3)
	{
		sigma_AOA = sigma_AOA - 15.5;
	}  // for calibration &   
	*/
	

	//cout << "sigma_AOA = " << sigma_AOA << endl;

	//getchar();

	//ASA_sigma = pow(10, AoA_mu + AoA_sigma * cross_corr);

	double C;


	if (TYPE == 1)//InH
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.434 * (0.9275 + 0.0439*K_factor - 0.0071*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.501;

		}

	}
	else if (TYPE == 2)
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.273;

		}

	}
	else if (TYPE == 3)
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.289;

		}

	}
	else if (TYPE == 4)
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.123 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.090;

		}

	}
	else if (TYPE == 5)
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.211 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.190;

		}

	}
	else if (TYPE == 6)   //////// 
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.273;

		}

	}
	else if (TYPE == 7)
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.289;

		}

	}
	else if (TYPE == 8)
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.289;

		}

	}



	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 51 ~ 


	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}




	/*
	AOA = new double[MAX_NUM_CLUSTERS];
	AOD = new double[MAX_NUM_CLUSTERS];
	*/
	AOA = new double[MAX_NUM_CLUSTERS];
	AOD = new double[MAX_NUM_CLUSTERS];

	double first_AOD;
	double first_AOA;

	double X1_AOD;
	double Y1_AOD;
	double X1_AOA;
	double Y1_AOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		//double Y_AOA = sigma_AOA / 7. * randnum.n();
		//double Y_AOD = sigma_AOD / 7. * randnum.n();

		double Y_AOA = normal(0, pow(sigma_AOA / 7. , 2.));
		double Y_AOD = normal(0, pow(sigma_AOD / 7. , 2.));

		if (i == 0)
		{
			X1_AOD = X;
			Y1_AOD = Y_AOD;

			X1_AOA = X;
			Y1_AOA = Y_AOA;
		}

		if (power[i] == 0.)
		{
			AOD[i] = 0.;
			AOA[i] = 0.;

		}
		else
		{
			if (TYPE == 1)
			{
				AOD[i] = -1 * sigma_AOD * log(power[i] / strongest_power) / C;
				AOA[i] = -1 * sigma_AOA * log(power[i] / strongest_power) / C;

				/*
				cout << AOA[i] << endl;
				cout << AOD[i] << endl;
				getchar();
				*/

			}
			else
			{
				AOD[i] = 2. * (sigma_AOD / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;
				AOA[i] = 2. * (sigma_AOA / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;

				/*
				cout << AOA[i] << endl;
				cout << AOD[i] << endl;
				getchar();
				*/
			}
		}

		if (i == 0)
		{
			first_AOD = AOD[0];
			first_AOA = AOA[0];
		}

		if (LOS == 1) // LOS
		{

			LOCATION vector;

			// bs_idx, ms_idx  ε ϵ ؼ Ʒ ģ.
			double sector_idx;
			
			/*
			cout << "ms = " << self_ms_idx << endl;
			cout << "sector = " << sector_idx << endl;
			getchar();
			*/

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else if (self_ms_idx % 30 < 30)
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);

			/*
			cout << angle << endl;
			getchar();
			*/


			AOD[i] = X * AOD[i] + Y_AOD - (X1_AOD * first_AOD + Y1_AOD - angle); //
			AOA[i] = X * AOA[i] + Y_AOA - (X1_AOA * first_AOA + Y1_AOA - angle);

		}
		else if (LOS == 0) // NLOS
		{

			LOCATION vector;

			// bs_idx, ms_idx  ε ϵ ؼ Ʒ ģ.
			double sector_idx;

			/*
			cout << "ms = " << self_ms_idx << endl;
			cout << "sector = " << sector_idx << endl;
			getchar();
			*/

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);

			//cout << angle << endl;
			//getchar();

			AOD[i] = X * AOD[i] + Y_AOD + angle; 
			AOA[i] = X * AOA[i] + Y_AOA + angle;

		}


	}

	/*
	cout << AOA[0] << endl;
	cout << AOD[0] << endl;
	cout << AOA[1] << endl;
	cout << AOD[1] << endl;
	getchar();
	*/
}

void CHANNEL::Set_ZOAZOD()
{

	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- ZOA, ZOD
	///
	/////////////////////////////////////////////////

	double sigma_ZOD = pow(10, normal(mu_ZSD, sigma_ZSD));
	double sigma_ZOA = pow(10, normal(mu_ZSA, sigma_ZSA));

	double C = 1;

	if (TYPE == 6)   //////// 
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.184;

		}

	}
	else if (TYPE == 7)
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.178;

		}

	}
	else if (TYPE == 8)
	{
		if (LOS == 1) ///// LOS
		{
			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);

		}
		else if (LOS == 0)   /////////// NLOS
		{
			C = 1.178;

		}

	}



	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////
	////// IMT 2020 EVAL Page 52 ~ 

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}



	ZOD = new double[MAX_NUM_CLUSTERS];
	ZOA = new double[MAX_NUM_CLUSTERS];

	double first_ZOD;
	double first_ZOA;

	double X1_ZOD;
	double Y1_ZOD;
	double X1_ZOA;
	double Y1_ZOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		double Y_ZOA = normal(0, pow(sigma_ZOA / 7., 2.)); //
		double Y_ZOD = normal(0, pow(sigma_ZOD / 7., 2.));

		if (i == 0)
		{
			X1_ZOD = X;
			Y1_ZOD = Y_ZOD;

			X1_ZOA = X;
			Y1_ZOA = Y_ZOA;
		}

		if (power[i] == 0.)
		{
			ZOD[i] = 0.;
			ZOA[i] = 0.;

		}
		else
		{
			
			ZOD[i] = -1 * sigma_ZOD * log(power[i] / strongest_power) / C;
			ZOA[i] = -1 * sigma_ZOA * log(power[i] / strongest_power) / C;
	
		}

		if (i == 0)
		{
			first_ZOD = ZOD[0];
			first_ZOA = ZOA[0];
		}




		if (LOS == 1) // LOS
		{

			LOCATION vector;

			// bs ms  angle 
			double sector_idx;

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else if (self_ms_idx % 30 < 30)
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);

			ZOA[i] = (X * ZOA[i] + Y_ZOA) - (X1_ZOA * first_ZOA + Y1_ZOA - angle);
			ZOD[i] = (X * ZOD[i] + Y_ZOD) - (X1_ZOD * first_ZOD + Y1_ZOD - angle);


			

		}
		else if (LOS == 0) // NLOS
		{

			LOCATION vector;

			// bs ms  angle 
			double sector_idx;

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);



			if (Indoor == 1)
			{
				ZOA[i] = X * ZOA[i] + Y_ZOA + 90.;                    // indoor϶ angle 90  (36.873)
				ZOD[i] = X * ZOD[i] + Y_ZOA + angle + mu_offset_ZOD; //̰͸ ܷ ٸ //߿ ¾ޱ۵  ٸǷ ؾѴ
			}
			else
			{
				ZOA[i] = X * ZOA[i] + Y_ZOA + angle;
				ZOD[i] = X * ZOD[i] + Y_ZOA + angle + mu_offset_ZOD; //̰͸ ܷ ٸ //߿ ¾ޱ۵  ٸǷ ؾѴ
			}

		}


	}

	



}

void CHANNEL::Set_InitialPhase()
{
	/////////////////////////////////////////////////
	///
	/// Small Scale parameter -- initial phase
	///
	/////////////////////////////////////////////////
	
	
	random_phase_vv = new double *[MAX_NUM_CLUSTERS];
	random_phase_vh = new double *[MAX_NUM_CLUSTERS];
	random_phase_hv = new double *[MAX_NUM_CLUSTERS];
	random_phase_hh = new double *[MAX_NUM_CLUSTERS];

	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		random_phase_vv[i] = new double[MAX_NUM_RAYS];
		random_phase_vh[i] = new double[MAX_NUM_RAYS];
		random_phase_hv[i] = new double[MAX_NUM_RAYS];
		random_phase_hh[i] = new double[MAX_NUM_RAYS];
	}
	



	
	for (int i = 0; i < MAX_NUM_CLUSTERS; i++)
	{
		for (int j = 0; j < MAX_NUM_RAYS; j++)
		{

			random_phase_vv[i][j] = 360. * randnum.u() - 180.;
			random_phase_vh[i][j] = 360. * randnum.u() - 180.;
			random_phase_hv[i][j] = 360. * randnum.u() - 180.;
			random_phase_hh[i][j] = 360. * randnum.u() - 180.;

		}
	}
	


	random_phase_vv_LOS = 360. * randnum.u() - 180.;
	random_phase_hh_LOS = 360. * randnum.u() - 180.;
}

void CHANNEL::Set_SUBCLUSTER()
{
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////
	////// offset_angle 
	///////////////////////////////////

	offset_angle = new double *[MAX_NUM_CLUSTERS];

	for (int path_idx = 0; path_idx < MAX_NUM_CLUSTERS; path_idx++)
	{
		offset_angle[path_idx] = new double[MAX_NUM_RAYS];

	}


	//////////////////////////////////////////////
	/////
	///// Set_subcluster 
	///// power  2 Ŭʹ 3 Ŭͷ  offset ش. ڵ   ƴ϶ 4 ߰ Ŭ(path) ߰ϴ , ray ٸ
	////
	/////////////////////////////////////////////

	delay[num_path] = delay[strongest_power_idx] + 0.000000005;
	delay[num_path + 1] = delay[strongest_power_idx] + 0.00000001;
	delay[num_path + 2] = delay[strongest_power_idx2] + 0.000000005;
	delay[num_path + 3] = delay[strongest_power_idx2] + 0.00000001;

	power[strongest_power_idx] = power[strongest_power_idx] * 10. / 20.;
	power[strongest_power_idx2] = power[strongest_power_idx2] * 10. / 20.;
	power[num_path] = power[strongest_power_idx] * 6. / 20.;
	power[num_path + 1] = power[strongest_power_idx] * 4. / 20.;
	power[num_path + 2] = power[strongest_power_idx2] * 6. / 20.;
	power[num_path + 3] = power[strongest_power_idx2] * 4. / 20.;

	AOA[num_path] = AOA[strongest_power_idx];
	AOA[num_path + 1] = AOA[strongest_power_idx];
	AOA[num_path + 2] = AOA[strongest_power_idx2];
	AOA[num_path + 3] = AOA[strongest_power_idx2];

	AOD[num_path] = AOD[strongest_power_idx];
	AOD[num_path + 1] = AOD[strongest_power_idx];
	AOD[num_path + 2] = AOD[strongest_power_idx2];
	AOD[num_path + 3] = AOD[strongest_power_idx2];

	for (int path_idx = 0; path_idx < num_path + 4; path_idx++)
	{
		if (path_idx == strongest_power_idx || path_idx == strongest_power_idx2) /// Subcluster No.1 (1,2,3,4,5,6,7,8,19,20 rays)
		{
			num_ray = 10;
			offset_angle[path_idx][0] = 0.0447;
			offset_angle[path_idx][1] = -0.0447;
			offset_angle[path_idx][2] = 0.1413;
			offset_angle[path_idx][3] = -0.1413;
			offset_angle[path_idx][4] = 0.2492;
			offset_angle[path_idx][5] = -0.2492;
			offset_angle[path_idx][6] = 0.3715;
			offset_angle[path_idx][7] = -0.3715;
			offset_angle[path_idx][8] = 2.1551;
			offset_angle[path_idx][9] = -2.1551;
		}

		else if (path_idx == num_path || path_idx == num_path + 2) /// Subcluster No.2 (9,10,11,12,17,18 rays)
		{
			num_ray = 6;
			offset_angle[path_idx][0] = 0.5129;
			offset_angle[path_idx][1] = -0.5129;
			offset_angle[path_idx][2] = 0.6797;
			offset_angle[path_idx][3] = -0.6797;
			offset_angle[path_idx][4] = 1.5195;
			offset_angle[path_idx][5] = -1.5195;
		}

		else if (path_idx == num_path + 1 || path_idx == num_path + 3) /// Subcluster No.3 (13,14,15,16 rays)
		{
			num_ray = 4;
			offset_angle[path_idx][0] = 0.8844;
			offset_angle[path_idx][1] = -0.8844;
			offset_angle[path_idx][2] = 1.1481;
			offset_angle[path_idx][3] = -1.1481;
		}

		else /// Not 2 strong cluster
		{
			num_ray = 20;
			offset_angle[path_idx][0] = 0.0447;
			offset_angle[path_idx][1] = -0.0447;
			offset_angle[path_idx][2] = 0.1413;
			offset_angle[path_idx][3] = -0.1413;
			offset_angle[path_idx][4] = 0.2492;
			offset_angle[path_idx][5] = -0.2492;
			offset_angle[path_idx][6] = 0.3715;
			offset_angle[path_idx][7] = -0.3715;
			offset_angle[path_idx][8] = 0.5129;
			offset_angle[path_idx][9] = -0.5129;
			offset_angle[path_idx][10] = 0.6797;
			offset_angle[path_idx][11] = -0.6797;
			offset_angle[path_idx][12] = 0.8844;
			offset_angle[path_idx][13] = -0.8844;
			offset_angle[path_idx][14] = 1.1481;
			offset_angle[path_idx][15] = -1.1481;
			offset_angle[path_idx][16] = 1.5195;
			offset_angle[path_idx][17] = -1.5195;
			offset_angle[path_idx][18] = 2.1551;
			offset_angle[path_idx][19] = -2.1551;
		}
	}

	num_path = num_path + 4;


}

void CHANNEL::Set_RMS_delay_spread()
{
	if (LOS == 0) //NLOS
	{
		delay_power = new double[MAX_NUM_CLUSTERS];
		power_ray_sum = new double[MAX_NUM_CLUSTERS];

		for (int i = 0; i < num_path; i++)
		{
			//delay_power[i] = num_ray * delay[i] * power[i];
			//power_ray_sum[i] = num_ray * power[i];

			delay_power[i] = delay[i] * power[i];
			power_ray_sum[i] = power[i];
		}

		double avg_delay;
		double delay_power_sum = 0;
		double power_sum = 0;


		for (int i = 0; i < num_path; i++)
		{
			delay_power_sum += delay_power[i];
			power_sum += power_ray_sum[i];
		}


		avg_delay = delay_power_sum / power_sum;


		RMS_delay_power = new double[MAX_NUM_CLUSTERS];


		for (int i = 0; i < num_path; i++)
		{
			RMS_delay_power[i] = (delay[i] - avg_delay) * (delay[i] - avg_delay) * power[i];
		}

		//double RMS_delay_spread = 0;
		double RMS_delay_power_sum = 0;

		for (int i = 0; i < num_path; i++)
		{
			RMS_delay_power_sum += RMS_delay_power[i];
		}

		RMS_delay_spread = sqrt(RMS_delay_power_sum / power_sum);
	}

	else if (LOS == 1) // LOS
	{
		delay_power = new double[MAX_NUM_CLUSTERS];
		power_ray_sum = new double[MAX_NUM_CLUSTERS];

		for (int i = 0; i < num_path; i++)
		{
			//delay_power[i] = num_ray * delay[i] * power[i];
			//power_ray_sum[i] = num_ray * power[i];

			delay_power[i] = delay_LOS[i] * power[i];
			power_ray_sum[i] = power[i];
		}

		double avg_delay;
		double delay_power_sum = 0;
		double power_sum = 0;


		for (int i = 0; i < num_path; i++)
		{
			delay_power_sum += delay_power[i];
			power_sum += power_ray_sum[i];
		}


		avg_delay = delay_power_sum / power_sum;


		RMS_delay_power = new double[MAX_NUM_CLUSTERS];


		for (int i = 0; i < num_path; i++)
		{
			RMS_delay_power[i] = (delay_LOS[i] - avg_delay) * (delay_LOS[i] - avg_delay) * power[i];
		}

		//double RMS_delay_spread = 0;
		double RMS_delay_power_sum = 0;

		for (int i = 0; i < num_path; i++)
		{
			RMS_delay_power_sum += RMS_delay_power[i];
		}

		RMS_delay_spread = sqrt(RMS_delay_power_sum / power_sum);

	}
	



}

void CHANNEL::Set_circular_angle_spread()
{
	//double mu_theta = 0;

	//3GPP 25.996 Annex A 
	
	AOA_power = new double[MAX_NUM_CLUSTERS];
	AOD_power = new double[MAX_NUM_CLUSTERS];
	power_ray_sum_circular = new double[MAX_NUM_CLUSTERS];

	for (int i = 0; i < num_path; i++)
	{
		//delay_power[i] = num_ray * delay[i] * power[i];
		//power_ray_sum[i] = num_ray * power[i];

		AOA_power[i] = AOA[i] * power[i]; 
		AOD_power[i] = AOD[i] * power[i];
		power_ray_sum_circular[i] = power[i];
	}

	double mu_theta_AOA;
	double mu_theta_AOD;
	double AOA_power_sum = 0;
	double AOD_power_sum = 0;
	double power_sum = 0;

	// AOA AOD degree
	
	for (int i = 0; i < num_path; i++)
	{
		AOA_power_sum += AOA_power[i];
		AOD_power_sum += AOD_power[i];
		power_sum += power_ray_sum_circular[i];
	}

	mu_theta_AOA = AOA_power_sum / power_sum;  // ̰͵ degreeε
	mu_theta_AOD = AOD_power_sum / power_sum;

	/*
	cout << "A = " << mu_theta_AOA << endl;
	cout << "D = " << mu_theta_AOD << endl;
	cout << pi << endl;
	getchar();
	*/



	//Calibration  ߰ 
	if (mu_theta_AOA < -180.) // pi Ǿִµ degree  => -180 ~ 180 ִ°
	{
		mu_theta_AOA = 2. * 180. + mu_theta_AOA;
	}
	else if (mu_theta_AOA > 180.)
	{
		mu_theta_AOA = mu_theta_AOA - 2. * 180.;
	}
	else
	{
		mu_theta_AOA = mu_theta_AOA;

	}

	if (mu_theta_AOD < -180.)
	{
		mu_theta_AOD = 2. * 180. + mu_theta_AOD;
	}
	else if (mu_theta_AOD > 180.)
	{
		mu_theta_AOD = mu_theta_AOD - 2. * 180.;
	}
	else
	{
		mu_theta_AOD = mu_theta_AOD;

	}


	theta_n_m_mu_AOA = new double[MAX_NUM_CLUSTERS];
	theta_n_m_mu_AOD = new double[MAX_NUM_CLUSTERS];

	for (int i = 0; i < num_path; i++)
	{
		if ((AOA[i] - mu_theta_AOA) < -180. ) // pi -> 180
		{
			theta_n_m_mu_AOA[i] = 2. * 180. + (AOA[i] - mu_theta_AOA);
		}
		else if (((AOA[i] - mu_theta_AOA) >= -180.) && ((AOA[i] - mu_theta_AOA) <= 180.))
		{
			theta_n_m_mu_AOA[i] = (AOA[i] - mu_theta_AOA);
		}
		else if ((AOA[i] - mu_theta_AOA) > 180.)
		{
			theta_n_m_mu_AOA[i] = 2. * 180. - (AOA[i] - mu_theta_AOA);
		}
		
	}

	for (int i = 0; i < num_path; i++)
	{
		if ((AOD[i] - mu_theta_AOD) < -180.)
		{
			theta_n_m_mu_AOD[i] = 2. * 180. + (AOD[i] - mu_theta_AOD);
		}
		else if (((AOD[i] - mu_theta_AOD) >= -180.) && ((AOD[i] - mu_theta_AOD) <= 180.))
		{
			theta_n_m_mu_AOD[i] = (AOD[i] - mu_theta_AOD);
		}
		else //if ((AOD[i] - mu_theta_AOD) > 180.)
		{
			theta_n_m_mu_AOD[i] = 2. * 180. - (AOD[i] - mu_theta_AOD);
		}

	}

	theta_power_AOA = new double[MAX_NUM_CLUSTERS];
	theta_power_AOD = new double[MAX_NUM_CLUSTERS];

	for (int i = 0; i < num_path; i++)
	{
		theta_power_AOA[i] = theta_n_m_mu_AOA[i] * theta_n_m_mu_AOA[i] * power[i];
		theta_power_AOD[i] = theta_n_m_mu_AOD[i] * theta_n_m_mu_AOD[i] * power[i];
	}

	double theta_power_sum_AOA = 0;
	double theta_power_sum_AOD = 0;

	for (int i = 0; i < num_path; i++)
	{
		theta_power_sum_AOA += theta_power_AOA[i];
		theta_power_sum_AOD += theta_power_AOD[i];
	}

	circular_angle_spread_AOA = sqrt(theta_power_sum_AOA / power_sum);
	circular_angle_spread_AOD = sqrt(theta_power_sum_AOD / power_sum);


}

void CHANNEL::Kill_Memory(void)
{
	
	for (int path_idx = 0; path_idx < MAX_NUM_CLUSTERS; path_idx++)
	{
		delete[] random_phase_vv[path_idx];
		delete[] random_phase_vh[path_idx];
		delete[] random_phase_hv[path_idx];
		delete[] random_phase_hh[path_idx];
		delete[] offset_angle   [path_idx];
	}

	delete[] random_phase_vv;
	delete[] random_phase_vh;
	delete[] random_phase_hv;
	delete[] random_phase_hh;
	delete[] offset_angle;


	delete[] delay;
	delete[] delay_LOS;

	delete[] power;
	delete[] power_LOS;
	delete[] power_NLOS;

	delete[] AOA;
	delete[] AOD;

	if (CH_CAL == 0)
	{
		delete[] AOA_LOS;
		delete[] AOD_LOS;

		delete[] AOA_NLOS;
		delete[] AOD_NLOS;
	}


	if (TYPE == 6 || TYPE == 7 || TYPE == 8 || TYPE == 11 || TYPE == 12 || TYPE == 13 || TYPE == 14 || TYPE == 15 || TYPE == 16 || TYPE == 17 || TYPE == 18)
	{
		delete[] ZOA;
		delete[] ZOD;

		if (CH_CAL == 0)
		{
			delete[] ZOA_LOS;
			delete[] ZOD_LOS;

			delete[] ZOA_NLOS;
			delete[] ZOD_NLOS;
		}


	}

	delete[] delay_power;
	delete[] power_ray_sum;
	delete[] RMS_delay_power;

	delete[] AOA_power;
	delete[] AOD_power;
	delete[] power_ray_sum_circular;


	delete[] theta_n_m_mu_AOA;
	delete[] theta_n_m_mu_AOD;

	delete[] theta_power_AOA;
	delete[] theta_power_AOD;




	if (CHIR == NULL)
	{

	}
	else
	{
		/*
		for (int u = 0; u < NUM_TX; u++)
		{
			for (int s = 0; s < NUM_RX; s++)
			{
				delete[] CHIR[u][s];
			}
		}

		for (int u = 0; u < NUM_TX; u++)
		{
			delete[] CHIR[u];
		}

		delete[] CHIR;
		*/
	}






}


void CHANNEL::Precalculate(int _time_idx )  // ̸ ϴ CH Ʈ κ
{
	//// =============¿=================================================//
	//ofstream CHIR_Precalc("Precalc_CHIR.dat");
	//CHIR_Precalc << "time_idx" << "\t" << "LOS" << "\t" << "INDOOR" << "\t" << "V_ANGLE_MAPPING" << "\t" << "NUM_TX" << "\t" << "NUM_RX" << "\t" << "NUM_CLUSTER" << "\t" << "CHIR" << endl;
	//// =============¿=================================================//

	if (TYPE == 1 || TYPE == 2 || TYPE == 3 || TYPE == 4 || TYPE == 5)
	{
		//ms_idx = _ms_idx;

		double ray_AOD = 0.; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
		double ray_AOA = 0.;
		double doppler_freq = 0.;


		//for (int time = 0; time < precalculate_time; time++)
		//{
			for (int los = 0; los < 2; los++)
			{
				for (int indoor = 0; indoor < 2; indoor++)
				{
					for (int v = 0; v < anglev_mapping_value; v++)
					{
						for (int u = 0; u < NUM_TX; u++)
						{
							for (int s = 0; s < NUM_RX; s++)
							{
								for (int i = 0; i < num_path; i++)
								{
									for (int j = 0; j < num_ray; j++)
									{
										if (los == 0)
										{
											ray_AOD = AOD_NLOS[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
											ray_AOA = AOA_NLOS[i] + cluster_ASA * offset_angle[i][j];
										}
										else if (los == 1)
										{
											ray_AOD = AOD_LOS[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
											ray_AOA = AOA_LOS[i] + cluster_ASA * offset_angle[i][j];
										}

										// indoor  user_speed ٸ ġ
										doppler_freq = (user_speed * 1000. / 3600.) * cos((ray_AOA - (2. * 180. * randnum.u() - 180.))*pi / 180) / Wavelength;
										
										

										double xpr = pow(10, XPR / 10);


										// angle_v  ̾Ƽ 10  ϳ ϵ ġ
										//   angle_v = (10 20 30 40 ...)  ϳ  ..


										//double angle_v = atan2((bs_height - ms_height), distance) * 180 / pi;

										//double angle_v = atan2((bs_height - ms_height), min_distance) * 180 / pi;
										
										//double angle_v = 10. * (anglev_mapping_value + 1.);

										double angle_v = ((MAX_v_angle / anglev_mapping_value) * (v + 1)) * randnum.u();

										if (angle_v < ((MAX_v_angle / anglev_mapping_value) * v))
										{
											angle_v = angle_v + ((MAX_v_angle / anglev_mapping_value) * v);
										}


										///////////F_tx_v
										F_tx_v = 12. * (angle_v - ANGLE_tilt)*(angle_v - ANGLE_tilt) / (15. * 15.); // 15 = pi_3dB // vertical(elevation) antenna pattern
										if (F_tx_v > 20.)
										{
											F_tx_v = 20.;
										}

										////////////F_tx_h
										F_tx_h = 12. * ray_AOD*ray_AOD / (70. * 70.); // 70 = theta_3dB // horizental antenna pattern
										if (F_tx_h > 20.)
										{
											F_tx_h = 20.;
										}

										//////////F_rx_v
										F_rx_v = 0.;

										//////////F_rx_h
										F_rx_h = 0.;

										F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
										F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
										F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
										F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



										if (TYPE == 1)
										{
											F_vv = 1;
											F_vh = 1;
											F_hv = 1;
											F_hh = 1;
										}



										complex <double>jay(0, 1);
										complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv[i][j]) + sqrt(1 / xpr)*F_hv*exp(jay*random_phase_hv[i][j]) + sqrt(1 / xpr)*F_vh*exp(jay*random_phase_vh[i][j]) + F_hh*exp(jay*random_phase_hh[i][j]);


										if (los == 1)  // LOS
										{
											CHIR_precalc[los][indoor][v][u][s][i] += sqrt(power_LOS[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180.) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180.) / Wavelength) * exp(jay * 2. * pi * doppler_freq * (double)(_time_idx / 1000.));
											// d_tx  uniform distance between tx element (m.2135-1)
										}
										else           // NLOS
										{
											CHIR_precalc[los][indoor][v][u][s][i] += sqrt(power_NLOS[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180.) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180.) / Wavelength) * exp(jay * 2. * pi * doppler_freq * (double)(_time_idx / 1000.));
											// d_tx  uniform distance between tx element (m.2135-1)
										}


										if (los == 1)
										{
											double k_factor = pow(10, K_factor / 10);

											//CHIR_precalculate[time][los][indoor][v][u][s][i] *= sqrt(1 / (k_factor + 1));
											CHIR_precalc[los][indoor][v][u][s][i] *= sqrt(1 / (k_factor + 1));
											
											double ray_delay = delay[0];
											double ray_AOD_los = AOD_LOS[0];
											double ray_AOA_los = AOA_LOS[0];

											// ʿ
											doppler_freq = (user_speed * 1000. / 3600.) * cos(ray_AOA_los - (2. * 180. * randnum.u() - 180.)) / Wavelength;

											///////////F_tx_v
											F_tx_v = 12. * (angle_v - ANGLE_tilt)*(angle_v - ANGLE_tilt) / (15. * 15.);
											if (F_tx_v > 20.)
											{
												F_tx_v = 20.;
											}

											////////////F_tx_h
											F_tx_h = 12. * ray_AOD_los * ray_AOD_los / (70. * 70.);
											if (F_tx_h > 20.)
											{
												F_tx_h = 20.;
											}


											//////////F_rx_v
											F_rx_v = 0.;

											//////////F_rx_h
											F_rx_h = 0.;

											F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
											F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
											F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
											F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



											if (TYPE == 1)
											{
												F_vv = 1;
												F_vh = 1;
												F_hv = 1;
												F_hh = 1;
											}


											// LOS϶ delta(n-1) * .... ִ° ִ. i = 0 ϶ ִ°.. i  cluster index
											if (i == 0)  
											{
												//CHIR_precalculate[time][los][indoor][v][u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD_los*pi / 180) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA_los*pi / 180) / Wavelength)* exp(jay*2.*pi*doppler_freq * (double)(time/1000.));
												CHIR_precalc[los][indoor][v][u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD_los*pi / 180.) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA_los*pi / 180.) / Wavelength)* exp(jay*2.*pi*doppler_freq * (double)(_time_idx / 1000.));
											}
										}
									}
									//// =============¿=================================================//
									//CHIR_Precalc << time << "\t" << los << "\t" << indoor << "\t" << v << "\t" << u << "\t" << s << "\t" << i << "\t" << CHIR_precalc[time][los][indoor][v][u][s][i] << endl;
									//// =============¿=================================================//
								}
							}
						}
					}
				}
			}

			//cout << "CHIR Time = " << time + 1 << "ms" << endl;

		//}
	}



	///////////////////////////
	///
	/// 3D channel model
	///
	///////////////////////////
	else
	{
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//ms_idx = _ms_idx;

		double ray_AOD = 0.; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
		double ray_AOA = 0.;
		double doppler_freq = 0.;
		double ray_ZOA = 0.;
		double ray_ZOD = 0.;


		//for (int time = 0; time < precalculate_time; time++)
		//{
			for (int los = 0; los < 2; los++)
			{
				for (int indoor = 0; indoor < 2; indoor++)
				{
					for (int v = 0; v < anglev_mapping_value; v++)
					{
						for (int u = 0; u < NUM_TX; u++)
						{
							for (int s = 0; s < NUM_RX; s++)
							{
								for (int i = 0; i < num_path; i++)
								{
									for (int j = 0; j < num_ray; j++)
									{
										if (los == 0)
										{
											ray_AOD = AOD_NLOS[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
											ray_AOA = AOA_NLOS[i] + cluster_ASA * offset_angle[i][j];

											ray_ZOA = ZOA_NLOS[i] + cluster_ZSA * offset_angle[i][j];
											ray_ZOD = ZOD_NLOS[i] + (3 / 8) * pow(10, mu_ZSD) * offset_angle[i][j];
										}
										else if (los == 1)
										{
											ray_AOD = AOD_LOS[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
											ray_AOA = AOA_LOS[i] + cluster_ASA * offset_angle[i][j];

											ray_ZOA = ZOA_LOS[i] + cluster_ZSA * offset_angle[i][j];
											ray_ZOD = ZOD_LOS[i] + (3 / 8) * pow(10, mu_ZSD) * offset_angle[i][j];
										}

										// indoor  user_speed ٸ ġ
										doppler_freq = (user_speed * 1000. / 3600.) * (sin(ray_ZOA * pi / 180) * cos(ray_AOA*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180) * cos((2. * 180. * randnum.u() - 180.)*pi / 180))
											+ (user_speed * 1000. / 3600.) * (sin(ray_ZOA*pi / 180) * sin(ray_AOA*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180))
											+ (user_speed * 1000. / 3600.) * (cos(ray_ZOA*pi / 180) * cos((2. * 180. * randnum.u() - 180.)*pi / 180));

										doppler_freq = doppler_freq / Wavelength;

										double xpr = pow(10, XPR / 10);

										///////////F_tx_v

										double ant_ZOD_v = 12. * (ray_ZOD - 90)*(ray_ZOD - 90) / (65. * 65.);
										double ant_AOD_v = 12. * (ray_AOD - 90)*(ray_AOD - 90) / (65. * 65.);

										F_tx_v = -MIN(-(ant_ZOD_v + ant_AOD_v), 30.);

										////////////F_tx_h

										double ant_ZOD_h = 12. * (ray_ZOD)*(ray_ZOD) / (65. * 65.);
										double ant_AOD_h = 12. * (ray_AOD)*(ray_AOD) / (65. * 65.);

										F_tx_h = -MIN(-(ant_ZOD_h + ant_AOD_h), 30.);

										//////////F_rx_v
										F_rx_v = 0.;

										//////////F_rx_h
										F_rx_h = 0.;

										F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
										F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
										F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
										F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);


										complex <double>jay(0, 1);
										complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv[i][j]) + sqrt(1 / xpr)*F_hv*exp(jay*random_phase_hv[i][j]) + sqrt(1 / xpr)*F_vh*exp(jay*random_phase_vh[i][j]) + F_hh*exp(jay*random_phase_hh[i][j]);

										if (los == 1)  // LOS
										{
											CHIR_precalc[los][indoor][v][u][s][i] += sqrt(power_LOS[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180.) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180.) / Wavelength) * exp(jay * 2. * pi * doppler_freq * (double)(_time_idx / 1000.));

											// d_tx  uniform distance between tx element (m.2135-1)
											//d_tx, d_rx  ̻ϴ. ڵ带  d_tx = 10 * wavelength * num_tx_antennaε // num_tx_antenna  ʴ(ڵ尡), ݿ ʿϴ // ߴ

											//⼭ d_tx location vector of tx antenna element (36.873) // m.2135-1 Ȱ ϵ سµ ٸ ִ(Ȯġ  κ..)
											//  exp ִ κ  κ ray_AOD  Ȯ
										}
										else           // NLOS 
										{
											CHIR_precalc[los][indoor][v][u][s][i] += sqrt(power_NLOS[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180.) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180.) / Wavelength) * exp(jay * 2. * pi * doppler_freq * (double)(_time_idx / 1000.));

											// d_tx  uniform distance between tx element (m.2135-1)
											//d_tx, d_rx  ̻ϴ. ڵ带  d_tx = 10 * wavelength * num_tx_antennaε // num_tx_antenna  ʴ(ڵ尡), ݿ ʿϴ // ߴ

											//⼭ d_tx location vector of tx antenna element (36.873) // m.2135-1 Ȱ ϵ سµ ٸ ִ(Ȯġ  κ..)
											//  exp ִ κ  κ ray_AOD  Ȯ
										}



										if (los == 1)
										{
											double k_factor = pow(10, K_factor / 10);

											//CHIR_precalculate[time][los][indoor][v][u][s][i] *= sqrt(1 / (k_factor + 1));

											CHIR_precalc[los][indoor][v][u][s][i] *= sqrt(1 / (k_factor + 1));

											double ray_delay = delay[0];
											double ray_AOD_los = AOD_LOS[0];
											double ray_AOA_los = AOA_LOS[0];

											double ray_ZOA_los = ZOA_LOS[0];
											double ray_ZOD_los = ZOD_LOS[0];

											// ʿ
											// indoor  user_speed ٸ ġ
											doppler_freq = user_speed * (sin(ray_ZOA * pi / 180) * cos(ray_AOA*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180) * cos((2. * 180. * randnum.u() - 180.)*pi / 180))
												+ user_speed * (sin(ray_ZOA*pi / 180) * sin(ray_AOA*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180) * sin((2. * 180. * randnum.u() - 180.)*pi / 180))
												+ user_speed * (cos(ray_ZOA*pi / 180) * cos((2. * 180. * randnum.u() - 180.)*pi / 180));

											doppler_freq = doppler_freq / Wavelength;

											///////////F_tx_v

											double ant_ZOD_v = 12. * (ray_ZOD - 90)*(ray_ZOD - 90) / (65. * 65.);
											double ant_AOD_v = 12. * (ray_AOD - 90)*(ray_AOD - 90) / (65. * 65.);

											F_tx_v = -MIN(-(ant_ZOD_v + ant_AOD_v), 30.);

											////////////F_tx_h

											double ant_ZOD_h = 12. * (ray_ZOD)*(ray_ZOD) / (65. * 65.);
											double ant_AOD_h = 12. * (ray_AOD)*(ray_AOD) / (65. * 65.);

											F_tx_h = -MIN(-(ant_ZOD_h + ant_AOD_h), 30.);

											//////////F_rx_v
											F_rx_v = 0.;

											//////////F_rx_h
											F_rx_h = 0.;

											F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
											F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
											F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
											F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



											// LOS϶ delta(n-1) * .... ִ° ִ. i =0 ϶ ִ°.. i  cluster index
											if (i == 0)
											{
												//CHIR_precalculate[time][los][indoor][v][u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD_los*pi / 180) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA_los*pi / 180) / Wavelength)* exp(jay*2.*pi*doppler_freq * (double)(time/1000.));
												CHIR_precalc[los][indoor][v][u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD_los*pi / 180.) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA_los*pi / 180.) / Wavelength)* exp(jay*2.*pi*doppler_freq * (double)(_time_idx / 1000.));
											}
										}
									}

									//// =============¿=================================================//
									//CHIR_Precalc << time << "\t" << los << "\t" << indoor << "\t" << v << "\t" << u << "\t" << s << "\t" << i << "\t" << CHIR_precalc[time][los][indoor][v][u][s][i] << endl;
									//// =============¿=================================================//
								}
							}
						}
					}
				}
			}

			//cout << "CHIR Time = " << time + 1 << "ms" << endl;
		//}
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	}

	//// =============¿=================================================//
	//CHIR_Precalc.close();
	//// =============¿=================================================//


}


void CHANNEL::Set_AOAAOD_LOS()
{
	/////////////////////////////////////////////////
	///
	/// FOR Pre-calculate CH - LOS case
	///
	/////////////////////////////////////////////////



	double sigma_AOD = pow(10, normal(mu_ASD, sigma_ASD));
	double sigma_AOA = pow(10, normal(mu_ASA, sigma_ASA));

	/*
	if (TYPE == 3)
	{
		sigma_AOA = sigma_AOA - 15.5;
	}  // for calibration &   
	*/


	double C;


	if (TYPE == 1)//InH
	{

			C = 1.434 * (0.9275 + 0.0439*K_factor - 0.0071*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 2)
	{

			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 3)
	{

			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 4)
	{

		C = 1.123 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

	}
	else if (TYPE == 5)
	{

			C = 1.211 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

	}
	else if (TYPE == 6)   //////// 
	{

			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);
	}
	else if (TYPE == 7)
	{

			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

	}
	else if (TYPE == 8)
	{
			C = 1.146 * (1.1035 - 0.028*K_factor - 0.002*K_factor*K_factor + 0.0001*K_factor*K_factor*K_factor);

	}
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}

	AOA = new double[MAX_NUM_CLUSTERS];
	AOD = new double[MAX_NUM_CLUSTERS];

	AOA_LOS = new double[MAX_NUM_CLUSTERS];
	AOD_LOS = new double[MAX_NUM_CLUSTERS];

	double first_AOD;
	double first_AOA;

	double X1_AOD;
	double Y1_AOD;
	double X1_AOA;
	double Y1_AOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		double Y_AOA = normal(0, pow(sigma_AOA / 7., 2.));
		double Y_AOD = normal(0, pow(sigma_AOD / 7., 2.));

		if (i == 0)
		{
			X1_AOD = X;
			Y1_AOD = Y_AOD;

			X1_AOA = X;
			Y1_AOA = Y_AOA;
		}

		if (power[i] == 0.)
		{
			AOD[i] = 0.;
			AOA[i] = 0.;

		}
		else
		{
			if (TYPE == 1)
			{
				AOD[i] = -1 * sigma_AOD * log(power[i] / strongest_power) / C;
				AOA[i] = -1 * sigma_AOA * log(power[i] / strongest_power) / C;

			}
			else
			{
				AOD[i] = 2. * (sigma_AOD / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;
				AOA[i] = 2. * (sigma_AOA / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;

			}
		}

		if (i == 0)
		{
			first_AOD = AOD[0];
			first_AOA = AOA[0];
		}

		if (true) // LOS
		{

			LOCATION vector;

			// bs_idx, ms_idx  ε ϵ ؼ Ʒ ģ.
			double sector_idx;



			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else if (self_ms_idx % 30 < 30)
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);


			AOD[i] = X * AOD[i] + Y_AOD - (X1_AOD * first_AOD + Y1_AOD - angle); //
			AOA[i] = X * AOA[i] + Y_AOA - (X1_AOA * first_AOA + Y1_AOA - angle);

		}

		AOA_LOS[i] = AOA[i];
		AOD_LOS[i] = AOD[i];

	}

	delete[] AOA;
	delete[] AOD;


}


void CHANNEL::Set_AOAAOD_NLOS()
{
	/////////////////////////////////////////////////
	///
	/// FOR Pre-calculate CH - NLOS case
	///
	/////////////////////////////////////////////////


	double sigma_AOD = pow(10, normal(mu_ASD, sigma_ASD));
	double sigma_AOA = pow(10, normal(mu_ASA, sigma_ASA));


	double C;


	if (TYPE == 1)//InH
	{

			C = 1.501;


	}
	else if (TYPE == 2)
	{

			C = 1.273;

	}
	else if (TYPE == 3)
	{

			C = 1.289;

	}
	else if (TYPE == 4)
	{

			C = 1.090;

	}
	else if (TYPE == 5)
	{

			C = 1.190;


	}
	else if (TYPE == 6)   //////// 
	{

			C = 1.273;

	}
	else if (TYPE == 7)
	{

			C = 1.289;

	}
	else if (TYPE == 8)
	{

			C = 1.289;


	}
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}


	AOA = new double[MAX_NUM_CLUSTERS];
	AOD = new double[MAX_NUM_CLUSTERS];

	AOA_NLOS = new double[MAX_NUM_CLUSTERS];
	AOD_NLOS = new double[MAX_NUM_CLUSTERS];

	double first_AOD;
	double first_AOA;

	double X1_AOD;
	double Y1_AOD;
	double X1_AOA;
	double Y1_AOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}


		double Y_AOA = normal(0, pow(sigma_AOA / 7., 2.));
		double Y_AOD = normal(0, pow(sigma_AOD / 7., 2.));

		if (i == 0)
		{
			X1_AOD = X;
			Y1_AOD = Y_AOD;

			X1_AOA = X;
			Y1_AOA = Y_AOA;
		}

		if (power[i] == 0.)
		{
			AOD[i] = 0.;
			AOA[i] = 0.;

		}
		else
		{
			if (TYPE == 1)
			{
				AOD[i] = -1 * sigma_AOD * log(power[i] / strongest_power) / C;
				AOA[i] = -1 * sigma_AOA * log(power[i] / strongest_power) / C;


			}
			else
			{
				AOD[i] = 2. * (sigma_AOD / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;
				AOA[i] = 2. * (sigma_AOA / 1.4) * sqrt(-log(power[i] / strongest_power)) / C;

			}
		}

		if (i == 0)
		{
			first_AOD = AOD[0];
			first_AOA = AOA[0];
		}


		if (true) // NLOS
		{

			LOCATION vector;

			// bs_idx, ms_idx  ε ϵ ؼ Ʒ ģ.
			double sector_idx;


			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);


			AOD[i] = X * AOD[i] + Y_AOD + angle;
			AOA[i] = X * AOA[i] + Y_AOA + angle;

		}

		AOA_NLOS[i] = AOA[i];
		AOD_NLOS[i] = AOD[i];


	}

	delete[] AOA;
	delete[] AOD;

}


void CHANNEL::Set_ZOAZOD_LOS()
{

	/////////////////////////////////////////////////
	///
	/// FOR Pre-calculate CH - LOS case
	///
	/////////////////////////////////////////////////

	double sigma_ZOD = pow(10, normal(mu_ZSD, sigma_ZSD));
	double sigma_ZOA = pow(10, normal(mu_ZSA, sigma_ZSA));

	double C = 1;

	if (TYPE == 6)   //////// 
	{

			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 7)
	{

			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);



	}
	else if (TYPE == 8)
	{
			C = 1.104 * (1.3086 + 0.0339*K_factor - 0.0077*K_factor*K_factor + 0.0002*K_factor*K_factor*K_factor);


	}
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}


	ZOD = new double[MAX_NUM_CLUSTERS];
	ZOA = new double[MAX_NUM_CLUSTERS];

	ZOD_LOS = new double[MAX_NUM_CLUSTERS];
	ZOA_LOS = new double[MAX_NUM_CLUSTERS];

	double first_ZOD;
	double first_ZOA;

	double X1_ZOD;
	double Y1_ZOD;
	double X1_ZOA;
	double Y1_ZOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		double Y_ZOA = normal(0, pow(sigma_ZOA / 7., 2.)); //
		double Y_ZOD = normal(0, pow(sigma_ZOD / 7., 2.));

		if (i == 0)
		{
			X1_ZOD = X;
			Y1_ZOD = Y_ZOD;

			X1_ZOA = X;
			Y1_ZOA = Y_ZOA;
		}

		if (power[i] == 0.)
		{
			ZOD[i] = 0.;
			ZOA[i] = 0.;

		}
		else
		{

			ZOD[i] = -1 * sigma_ZOD * log(power[i] / strongest_power) / C;
			ZOA[i] = -1 * sigma_ZOA * log(power[i] / strongest_power) / C;

		}

		if (i == 0)
		{
			first_ZOD = ZOD[0];
			first_ZOA = ZOA[0];
		}




		if (true) // LOS
		{

			LOCATION vector;

			// bs ms  angle 
			double sector_idx;

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else if (self_ms_idx % 30 < 30)
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);

			ZOA[i] = (X * ZOA[i] + Y_ZOA) - (X1_ZOA * first_ZOA + Y1_ZOA - angle);
			ZOD[i] = (X * ZOD[i] + Y_ZOD) - (X1_ZOD * first_ZOD + Y1_ZOD - angle);




		}

		ZOA_LOS[i] = ZOA[i];
		ZOD_LOS[i] = ZOD[i];



	}

	delete[] ZOA;
	delete[] ZOD;



}


void CHANNEL::Set_ZOAZOD_NLOS()
{

	/////////////////////////////////////////////////
	///
	/// FOR Pre-calculate CH - NLOS case
	///
	/////////////////////////////////////////////////

	double sigma_ZOD = pow(10, normal(mu_ZSD, sigma_ZSD));
	double sigma_ZOA = pow(10, normal(mu_ZSA, sigma_ZSA));

	double C = 1;

	if (TYPE == 6)   //////// 
	{
			C = 1.184;
	}
	else if (TYPE == 7)
	{
			C = 1.178;
	}
	else if (TYPE == 8)
	{
			C = 1.178;
	}
	//////////////////////////////////////////////////////// IMT-2020 /////////////////////////////////////////////////////////////////////////////////////////////////

	else if (TYPE == 11) // InH
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 12) // Dense_Urban
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}
	else if (TYPE == 13) // RMa
	{
		if (Channel_Model_Type == 0) /// Channel Model A
		{

		}
		else if (Channel_Model_Type == 1) // Channel Model B
		{

		}

	}


	ZOD = new double[MAX_NUM_CLUSTERS];
	ZOA = new double[MAX_NUM_CLUSTERS];

	ZOD_NLOS = new double[MAX_NUM_CLUSTERS];
	ZOA_NLOS = new double[MAX_NUM_CLUSTERS];

	double first_ZOD;
	double first_ZOA;

	double X1_ZOD;
	double Y1_ZOD;
	double X1_ZOA;
	double Y1_ZOA;

	for (int i = 0; i < num_path; i++)
	{

		double R = randnum.u();
		double X;
		if (R > 0.5)
		{
			X = 1;
		}
		else
		{
			X = -1;
		}

		double Y_ZOA = normal(0, pow(sigma_ZOA / 7., 2.)); //
		double Y_ZOD = normal(0, pow(sigma_ZOD / 7., 2.));

		if (i == 0)
		{
			X1_ZOD = X;
			Y1_ZOD = Y_ZOD;

			X1_ZOA = X;
			Y1_ZOA = Y_ZOA;
		}

		if (power[i] == 0.)
		{
			ZOD[i] = 0.;
			ZOA[i] = 0.;

		}
		else
		{

			ZOD[i] = -1 * sigma_ZOD * log(power[i] / strongest_power) / C;
			ZOA[i] = -1 * sigma_ZOA * log(power[i] / strongest_power) / C;

		}

		if (i == 0)
		{
			first_ZOD = ZOD[0];
			first_ZOA = ZOA[0];
		}




		if (true) // NLOS
		{

			LOCATION vector;

			// bs ms  angle 
			double sector_idx;

			if (self_ms_idx % 30 < 10)
			{
				sector_idx = 1;
			}
			else if (self_ms_idx % 30 < 20)
			{
				sector_idx = 2;
			}
			else
			{
				sector_idx = 3;
			}

			vector.x = bs_location.x + cos((30. + 120. * (sector_idx - 1.))*pi / 180.);
			vector.y = bs_location.y + sin((30. + 120. * (sector_idx - 1.))*pi / 180.);

			double angle = link[self_ms_idx].Get_angle(bs_location, ms_location, vector);



			if (Indoor == 1)
			{
				ZOA[i] = X * ZOA[i] + Y_ZOA + 90.;                    // indoor϶ angle 90  (36.873)
				ZOD[i] = X * ZOD[i] + Y_ZOA + angle + mu_offset_ZOD; //̰͸ ܷ ٸ //߿ ¾ޱ۵  ٸǷ ؾѴ
			}
			else
			{
				ZOA[i] = X * ZOA[i] + Y_ZOA + angle;
				ZOD[i] = X * ZOD[i] + Y_ZOA + angle + mu_offset_ZOD; //̰͸ ܷ ٸ //߿ ¾ޱ۵  ٸǷ ؾѴ
			}


		}

		ZOA_NLOS[i] = ZOA[i];
		ZOD_NLOS[i] = ZOD[i];

	}

	delete[] ZOA;
	delete[] ZOD;



}


void CHANNEL::Load_precalculate(double t, int _ms_idx, int _initial_time_offset)
{
	ms_idx = _ms_idx;


	int los_idx;
	int indoor_idx;
	int v_idx;
	
	

	if (LOS == 1)
	{
		los_idx = 1;
	}
	else
	{
		los_idx = 0;
	}

	if (Indoor == 1) //////// InH   ִ
	{
		indoor_idx = 1;
	}
	else
	{
		indoor_idx = 0;
	}


	if (TYPE == 1 || TYPE == 2 || TYPE == 3 || TYPE == 4 || TYPE == 5)
	{
		double angle_v = atan2((bs_height - ms_height), distance) * 180. / pi;

		//// 
		if (angle_v > MAX_v_angle - (MAX_v_angle/anglev_mapping_value))
		{
			v_idx = 2;
		}
		else if (angle_v < (MAX_v_angle / anglev_mapping_value))
		{
			v_idx = 0;
		}
		else
		{
			v_idx = 1;
		}
	}
	else
	{
		//v_idx = (int)(anglev_mapping_value * randnum.u());
		v_idx = 0;  // 3D channel model vertical angle , ZOA ϱ⶧
	}




	//if (TYPE == 1 || TYPE == 2 || TYPE == 3 || TYPE == 4 || TYPE == 5)
	//{
		//for (int u = 0; u < NUM_TX; u++)
		//{
			//for (int s = 0; s < NUM_RX; s++)
			//{
				//for (int i = 0; i < num_path; i++)
				//{
					//CHIR[u][s][i] = CHIR_precalc[(int)(_initial_time_offset + t) % precalculate_time][los_idx][indoor_idx][v_idx][u][s][i];
				//}
			//}
		//}
	//}

}


void CHANNEL::Update(double t, int _ms_idx)
{
	/*//////////////////////////////////////////////////////////
	// INPUT:
	// OUTPUT: CHIR
	// Notes: ð t  CHIR 
	*///////////////////////////////////////////////////////////

	//cout << atan2((bs_height - ms_height), distance) * 180 / pi << endl;
	//getchar();

	if (TYPE == 1 || TYPE == 2 || TYPE == 3 || TYPE == 4 || TYPE == 5)
	{
		ms_idx = _ms_idx;

		//CHIR = new complex <double>**[NUM_TX];

		for (int u = 0; u < NUM_TX; u++)
		{
			//CHIR[u] = new complex <double>*[NUM_RX];

			for (int s = 0; s < NUM_RX; s++)
			{
				//CHIR[u][s] = new complex <double>[MAX_NUM_CLUSTERS];

				for (int i = 0; i < num_path; i++)
				{
					//CHIR[u][s][i] = complex <double>(0, 0);


					for (int j = 0; j < num_ray; j++)
					{

						double ray_AOD = AOD[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
						double ray_AOA = AOA[i] + cluster_ASA * offset_angle[i][j];

						double doppler_freq = ms[ms_idx].speed* cos((ray_AOA - ms[ms_idx].moving_direction)*pi / 180) / Wavelength;

						double xpr = pow(10, XPR / 10);

						double angle_v = atan2((bs_height - ms_height), distance) * 180 / pi;

						//cout << "angle_v = " << angle_v << endl;

						///////////F_tx_v
						F_tx_v = 12. * (angle_v - ANGLE_tilt)*(angle_v - ANGLE_tilt) / (15. * 15.); // 15 = pi_3dB // vertical(elevation) antenna pattern
						if (F_tx_v > 20.)
						{
							F_tx_v = 20.;
						}

						////////////F_tx_h
						F_tx_h = 12. * ray_AOD*ray_AOD / (70. * 70.); // 70 = theta_3dB // horizental antenna pattern
						if (F_tx_h > 20.)
						{
							F_tx_h = 20.;
						}

						//////////F_rx_v
						F_rx_v = 0.;

						//////////F_rx_h
						F_rx_h = 0.;

						F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
						F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
						F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
						F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



						if (TYPE == 1)
						{
							F_vv = 1;
							F_vh = 1;
							F_hv = 1;
							F_hh = 1;
						}



						complex <double>jay(0, 1);
						complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv[i][j]) + sqrt(1 / xpr)*F_hv*exp(jay*random_phase_hv[i][j]) + sqrt(1 / xpr)*F_vh*exp(jay*random_phase_vh[i][j]) + F_hh*exp(jay*random_phase_hh[i][j]);

						CHIR[u][s][i] += sqrt(power[i] / num_ray) * antenna_gain * exp(jay * (d_tx * u) * 2. * pi * sin(ray_AOD*pi / 180) / Wavelength) * exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180) / Wavelength) * exp(jay * 2. * pi * doppler_freq * t / 1000.);

						// d_tx  uniform distance between tx element (m.2135-1)

						/*
						cout << "LOS = " << LOS << endl;
						cout << "ms_idx = " << ms_idx << endl;
						cout << "NUM_TX = " << u << endl;
						cout << "NUM_RX = " << s << endl;
						cout << "NUM_CLUSTERS = " << i << endl;
						cout << "CHIR = "  << CHIR[u][s][i] << endl;

						getchar();
						*/

					}
				}
			}
		}

		if (LOS == 1)
		{
			double k_factor = pow(10, K_factor / 10);

			for (int u = 0; u < NUM_TX; u++)
			{
				for (int s = 0; s < NUM_RX; s++)
				{
					for (int i = 0; i < num_path; i++)
					{
						CHIR[u][s][i] *= sqrt(1 / (k_factor + 1));
					}
				}
			}

			double ray_delay = delay[0];
			double ray_AOD = AOD[0];
			double ray_AOA = AOA[0];

			double doppler_freq = ms[ms_idx].speed * cos(ray_AOA - ms[ms_idx].moving_direction) / Wavelength;
			double xpr = pow(10, XPR / 10);

			double angle_v = atan2((bs_height - ms_height), distance) * 180 / pi;
			///////////F_tx_v
			F_tx_v = 12. * (angle_v - ANGLE_tilt)*(angle_v - ANGLE_tilt) / (15. * 15.);
			if (F_tx_v > 20.)
			{
				F_tx_v = 20.;
			}

			////////////F_tx_h
			F_tx_h = 12. * ray_AOD*ray_AOD / (70. * 70.);
			if (F_tx_h > 20.)
			{
				F_tx_h = 20.;
			}

			//////////F_rx_v
			F_rx_v = 0.;

			//////////F_rx_h
			F_rx_h = 0.;

			F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
			F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
			F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
			F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



			if (TYPE == 1)
			{
				F_vv = 1;
				F_vh = 1;
				F_hv = 1;
				F_hh = 1;
			}



			complex <double> jay(0, 1);
			complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv_LOS) + F_hh*exp(jay*random_phase_hh_LOS);
			for (int u = 0; u < NUM_TX; u++)
			{
				for (int s = 0; s < NUM_RX; s++)
				{
					CHIR[u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD*pi / 180) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180) / Wavelength)* exp(jay*2.*pi*doppler_freq*t / 1000.);


					/*
					cout << "LOS = " << LOS << endl;
					cout << "ms_idx = " << ms_idx << endl;
					cout << "NUM_TX = " << u << endl;
					cout << "NUM_RX = " << s << endl;
					//cout << "NUM_CLUSTERS = " << i << endl;
					cout << "CHIR = " << CHIR[u][s][0] << endl;

					getchar();
					*/


				}

			}


		}


	}



	/*
	double h_antgain = -MIN(12. * (h_angle / 65.)*(h_angle / 65.), 30.);
	double v_antgain = -MIN(12. * ((v_angle - 90) / 65.) * ((v_angle - 90) / 65.), 30.);

	return max_antgain - MIN(-(h_antgain + v_antgain), 30.);

	*/


	///////////////////////////
	///
	/// 3D channel model
	///
	///////////////////////////
	else
	{

		ms_idx = _ms_idx;

		//CHIR = new complex <double>**[NUM_TX];

		for (int u = 0; u < NUM_TX; u++)
		{
			//CHIR[u] = new complex <double>*[NUM_RX];

			for (int s = 0; s < NUM_RX; s++)
			{
				//CHIR[u][s] = new complex <double>[MAX_NUM_CLUSTERS];

				for (int i = 0; i < num_path; i++)
				{
					//CHIR[u][s][i] = complex <double>(0, 0);


					for (int j = 0; j < num_ray; j++)
					{

						double ray_AOD = AOD[i] + cluster_ASD * offset_angle[i][j]; //ZOD  ZOD[i] + (3/8) * pow(10,mu_ZSD) * offset_angle[i][j]
						double ray_AOA = AOA[i] + cluster_ASA * offset_angle[i][j];

						double ray_ZOA = ZOA[i] + cluster_ZSA * offset_angle[i][j];
						double ray_ZOD = ZOD[i] + (3 / 8) * pow(10, mu_ZSD) * offset_angle[i][j];

						// 3D Ʒ ʿµ, ZOA ZOD ⶧
						//double angle_v = atan2((bs_height - ms_height), distance) * 180 / pi;

						//double doppler_freq = ms[ms_idx].speed* cos((ray_AOA - ms[ms_idx].moving_direction)*pi / 180) / Wavelength;
						double doppler_freq = ms[ms_idx].speed * (sin(ray_ZOA * pi / 180) * cos(ray_AOA*pi / 180) * sin(ms[ms_idx].moving_direction*pi / 180) * cos(ms[ms_idx].moving_direction_azimuth*pi / 180))
							+ ms[ms_idx].speed * (sin(ray_ZOA*pi / 180) * sin(ray_AOA*pi / 180) * sin(ms[ms_idx].moving_direction*pi / 180) * sin(ms[ms_idx].moving_direction_azimuth*pi / 180))
							+ ms[ms_idx].speed * (cos(ray_ZOA*pi / 180) * cos(ms[ms_idx].moving_direction*pi / 180));

						doppler_freq = doppler_freq / Wavelength;

						double xpr = pow(10, XPR / 10);

						///////////F_tx_v

						double ant_ZOD_v = 12. * (ray_ZOD - 90)*(ray_ZOD - 90) / (65. * 65.);
						double ant_AOD_v = 12. * (ray_AOD - 90)*(ray_AOD - 90) / (65. * 65.);

						F_tx_v = -MIN(-(ant_ZOD_v + ant_AOD_v), 30.);


						////////////F_tx_h

						double ant_ZOD_h = 12. * (ray_ZOD)*(ray_ZOD) / (65. * 65.);
						double ant_AOD_h = 12. * (ray_AOD)*(ray_AOD) / (65. * 65.);

						F_tx_h = -MIN(-(ant_ZOD_h + ant_AOD_h), 30.);

						//////////F_rx_v
						F_rx_v = 0.;

						//////////F_rx_h
						F_rx_h = 0.;

						F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
						F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
						F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
						F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);



						complex <double>jay(0, 1);
						complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv[i][j]) + sqrt(1 / xpr)*F_hv*exp(jay*random_phase_hv[i][j]) + sqrt(1 / xpr)*F_vh*exp(jay*random_phase_vh[i][j]) + F_hh*exp(jay*random_phase_hh[i][j]);

						CHIR[u][s][i] += sqrt(power[i] / num_ray) * antenna_gain *exp(jay * (d_tx * u) * 2. * pi*sin(ray_AOD*pi / 180) / Wavelength)*exp(jay * (d_rx * s) * 2. * pi*sin(ray_AOA*pi / 180) / Wavelength)*exp(jay*2.*pi*doppler_freq*t / 1000.);

						//d_tx, d_rx  ̻ϴ. ڵ带  d_tx = 10 * wavelength * num_tx_antennaε // num_tx_antenna  ʴ(ڵ尡), ݿ ʿϴ // ߴ

						//⼭ d_tx location vector of tx antenna element (36.873) // m.2135-1 Ȱ ϵ سµ ٸ ִ(Ȯġ  κ..)



						//  exp ִ κ  κ ray_AOD  Ȯ
					}
				}
			}
		}

		if (LOS == 1)
		{
			double k_factor = pow(10, K_factor / 10);

			for (int u = 0; u < NUM_TX; u++)
			{
				for (int s = 0; s < NUM_RX; s++)
				{
					for (int i = 0; i < num_path; i++)
					{
						CHIR[u][s][i] *= sqrt(1 / (k_factor + 1));
					}
				}
			}

			double ray_delay = delay[0];
			double ray_AOD = AOD[0];
			double ray_AOA = AOA[0];

			double ray_ZOA = ZOA[0];
			double ray_ZOD = ZOD[0];

			// 3D Ʒ ʿµ, ZOA ZOD ⶧
			//double angle_v = atan2((bs_height - ms_height), distance) * 180 / pi;

			//double doppler_freq = ms[ms_idx].speed* cos((ray_AOA - ms[ms_idx].moving_direction)*pi / 180) / Wavelength;
			double doppler_freq = ms[ms_idx].speed * (sin(ray_ZOA * pi / 180) * cos(ray_AOA*pi / 180) * sin(ms[ms_idx].moving_direction*pi / 180) * cos(ms[ms_idx].moving_direction_azimuth*pi / 180))
				+ ms[ms_idx].speed * (sin(ray_ZOA*pi / 180) * sin(ray_AOA*pi / 180) * sin(ms[ms_idx].moving_direction*pi / 180) * sin(ms[ms_idx].moving_direction_azimuth*pi / 180))
				+ ms[ms_idx].speed * (cos(ray_ZOA*pi / 180) * cos(ms[ms_idx].moving_direction*pi / 180));

			doppler_freq = doppler_freq / Wavelength;

			double xpr = pow(10, XPR / 10);

			///////////F_tx_v

			double ant_ZOD_v = 12. * (ray_ZOD - 90)*(ray_ZOD - 90) / (65. * 65.);
			double ant_AOD_v = 12. * (ray_AOD - 90)*(ray_AOD - 90) / (65. * 65.);

			F_tx_v = -MIN(-(ant_ZOD_v + ant_AOD_v), 30.);


			////////////F_tx_h

			double ant_ZOD_h = 12. * (ray_ZOD)*(ray_ZOD) / (65. * 65.);
			double ant_AOD_h = 12. * (ray_AOD)*(ray_AOD) / (65. * 65.);

			F_tx_h = -MIN(-(ant_ZOD_h + ant_AOD_h), 30.);

			//////////F_rx_v
			F_rx_v = 0.;

			//////////F_rx_h
			F_rx_h = 0.;

			F_vv = pow(10., -(F_rx_v + F_tx_v) / 10.);
			F_vh = pow(10., -(F_rx_v + F_tx_h) / 10.);
			F_hv = pow(10., -(F_rx_h + F_tx_v) / 10.);
			F_hh = pow(10., -(F_rx_h + F_tx_h) / 10.);


			complex <double> jay(0, 1);
			complex <double> antenna_gain = F_vv*exp(jay*random_phase_vv_LOS) + F_hh*exp(jay*random_phase_hh_LOS);
			for (int u = 0; u < NUM_TX; u++)
			{
				for (int s = 0; s < NUM_RX; s++)
				{
					CHIR[u][s][0] += sqrt(k_factor / (k_factor + 1)) * antenna_gain *exp(jay * (d_tx*u) * 2. * pi*sin(ray_AOD*pi / 180) / Wavelength)*exp(jay * (d_rx*s) * 2. * pi*sin(ray_AOA*pi / 180) / Wavelength)* exp(jay*2.*pi*doppler_freq*t / 1000.);
				}
			}
		}

	}





}

